/****************************************************************************
 *
 * Copy right:   Qx.Chen jie
 * File name:    AppTaskGps.c
 * Description:  Gps处理任务和三轴数据处理及获取
 * History:      2021-03-07 2021-07-15
 * Version:      V3.0
 ****************************************************************************/
#include "AppTaskGps.h"
static QueueHandle_t norGpsHandle = NULL;
static osThreadId_t gpsTaskHandle = NULL;
static StaticTask_t gpsTask = NULL;
static UINT8 gpsTaskStack[GPS_TASK_STACK_SIZE];
extern void GsensorInit(void);
extern void GsensorI2CHandler(ARM_I2C_SignalEvent_t cb_event);
extern void GsensorI2CCallback(UINT32 event);

static process_gps             gProcess_Gps_Task = PROCESS_GPS_STATE_IDLE;
#define PROC_GPS_STATE_SWITCH(a)  (gProcess_Gps_Task = a)
UINT32 location_handle(char *in1);
void strdel(char * str,char c);
static void GpsTask(void* arg)
{
	/*三轴加速度初始化*/
	GsensorI2CHandler(GsensorI2CCallback);
    GsensorInit();

	if(GpsRecvHandle == NULL)
	{
		GpsRecvHandle = osMessageQueueNew(2,sizeof(GPSInfo*), NULL);
	}
	gpsReqMsg msg;
	GPSInfo *GpsInfoData; 
	char *p=NULL;
	const char *delim = "\n";
	char *databuffer[14];
	UINT32 speedtemp;
    UINT32 latitude;
	UINT32 longitude;
    UINT16 direction;
	UINT8  	GpsNoDataCounter = 0;
	UINT8	GpsResetCount = 0;
	PROC_GPS_STATE_SWITCH(PROCESS_GPS_STATE_INIT);
    while(1)
    {
        switch(gProcess_Gps_Task)
		{
			case PROCESS_GPS_STATE_INIT:
			{
				posGGAReset();
				osDelay(100);
				posGGAServiceStart(norGpsHandle);
				PROC_GPS_STATE_SWITCH(PROCESS_GPS_STATE_WORK);
				break;
			}
			case PROCESS_GPS_STATE_IDLE:
			{

				osDelay(100);
				if(gProcess_app==WORK)
				{
					PROC_GPS_STATE_SWITCH(PROCESS_GPS_STATE_WORK);
					break;
				}
				else
				{
					PROC_GPS_STATE_SWITCH(PROCESS_GPS_STATE_SLEEP);
					break;
				}
				break;
			}
			case PROCESS_GPS_STATE_WORK:
			{
				if(GpsNoDataCounter>=10 && GpsResetCount<=3)
				{
					posGGAReset();
					GpsResetCount++;
					GpsNoDataCounter = 0;
					osDelay(100);
					posGGAServiceStart(norGpsHandle);
				}
				osStatus_t ret = osMessageQueueGet(norGpsHandle, &msg, 0, 1000);
				if(ret==0)
				{
					if (msg.dataPtr)
            		{
						//char temp[] = "$GNRMC,082626.000,A,2939.91801,N,10637.09500,E,0.543,30.254,261120,,,A,V*17";
						GpsInfoData = malloc(sizeof(GPSInfo));
						memset(GpsInfoData,0x00,sizeof(GPSInfo));
						p = strtok(msg.dataPtr,delim);//将信息进行分割
						#ifdef USING_PRINTF1
							printf("\nP msgptr data:%s\r\n",p);
						#endif
						p = strtok(p,",");//只取第1行的信息RMC
						//p = strtok(temp,",");//模拟测试
						if (strcmp(p,"$GNRMC")==0)
						{
							UINT8 i=0;			
							while (p)
							{	
								databuffer[i]=p;
								p = strtok(NULL,",");
								i++;;
							}
							if (strcmp(databuffer[1],"V")==0|strcmp(databuffer[2],"V")==0)
							{
								GpsFlag = 0;
								memset(GpsInfoData,0x00,sizeof(GPSInfo));
								GpsNoDataCounter++;
							}
							else if (strcmp(databuffer[2],"A")==0)
							{
								GpsResetCount = 0;
								GpsInfoData->locateMark = 0x01;//有效，东经，北纬写定
								GpsInfoData->satelliteNum = 03;//卫星数目写入1
								GpsFlag = GpsInfoData->satelliteNum;
								strdel(databuffer[3],'.');
								strdel(databuffer[5],'.');
								strdel(databuffer[7],'.');
								speedtemp = atol(databuffer[7])*1852/1e5;//节换算单位，1节=1.852km每小时
								GpsInfoData->speed[0] = (speedtemp>>8)&0xFF;
								GpsInfoData->speed[1]= speedtemp&0xFF; 
								latitude =location_handle(databuffer[3]);
								GpsInfoData->latitude[0] = latitude>> 24;
								GpsInfoData->latitude[1] = latitude>> 16;
								GpsInfoData->latitude[2] = latitude>> 8;
								GpsInfoData->latitude[3] = latitude;
								longitude = location_handle(databuffer[5]);
								GpsInfoData->longitude[0] = longitude>>24;
								GpsInfoData->longitude[1] = longitude>>16;
								GpsInfoData->longitude[2] = longitude>>8;
								GpsInfoData->longitude[3] = longitude;

								GpsInfoData->altitude[0]  = 0x03;//海拔目前无输出
								GpsInfoData->altitude[1]  = 0xE8;
								if(speedtemp>=50)//大于5km/h才输出方位
								{
									direction = atol(databuffer[8]);
									GpsInfoData->direction[0] = direction>>8;
									GpsInfoData->direction[1] = direction;
								}
								else
								{
									GpsInfoData->direction[0] = 0xff;
									GpsInfoData->direction[1] = 0xff;
								}
								if(speedtemp>=30 && speedtemp<=1500)
								{
									AppDataInfo.appDataModify = true;
									AppDataInfo.AccMileage = speedtemp/36 + AppDataInfo.AccMileage;
									if(AppDataInfo.AccMileage>=0xfffffffe)
									{
										AppDataInfo.AccMileage = 0;
									}

								}
								GpsInfoData->AccMileage[0] = AppDataInfo.AccMileage>>24;
								GpsInfoData->AccMileage[1] = AppDataInfo.AccMileage>>16;
								GpsInfoData->AccMileage[2] = AppDataInfo.AccMileage>>8;
								GpsInfoData->AccMileage[3] = AppDataInfo.AccMileage;
							}
						}
						osMessageQueuePut(GpsRecvHandle,&GpsInfoData,0,1000);
					}
					if(msg.dataPtr)
						free(msg.dataPtr);
					msg.dataPtr=NULL;
				}
				PROC_GPS_STATE_SWITCH(PROCESS_GPS_STATE_IDLE);
				break;
			}
			case PROCESS_GPS_STATE_SLEEP:
			{
				posGGAServiceStop();
				while(TRUE)
				{
					osDelay(100);
					if(gProcess_app==WORK)
					{
						PROC_GPS_STATE_SWITCH(PROCESS_GPS_STATE_INIT);
						break;
					}
				}
				break;
			}
		}
    }
}
void AppTaskGpsInit(void *arg)
{

	if(norGpsHandle == NULL)
	{
		norGpsHandle = osMessageQueueNew(1,sizeof(gpsReqMsg), NULL);
		if(norGpsHandle == NULL)
			return;
	}
	if(gpsTaskHandle == NULL)
	{
		osThreadAttr_t task_attr;
		memset(&task_attr , 0 , sizeof(task_attr));
		task_attr.name = "GPS";
		task_attr.priority = osPriorityBelowNormal6;
		task_attr.cb_mem = &gpsTask;
		task_attr.cb_size = sizeof(StaticTask_t);
		task_attr.stack_mem = gpsTaskStack;
		task_attr.stack_size =GPS_TASK_STACK_SIZE;
		memset(& gpsTaskStack, 0xa5, GPS_TASK_STACK_SIZE);
		gpsTaskHandle = osThreadNew(GpsTask , NULL,&task_attr);
		if(gpsTaskHandle == NULL)
			return;
	}
}
void AppTaskGpsDeInit(void *arg)
{
    osThreadTerminate(gpsTaskHandle);
    gpsTaskHandle = NULL;
}
UINT32 location_handle(char *in1)
{
	UINT32 location_temp;
	UINT32 location_degree;
	UINT32 location_min;
	location_temp = atol(in1);
	location_degree = location_temp/(1e7);
	location_degree = location_degree*(1e6);
	location_min = location_temp-location_degree*10;
	location_min = location_min/6;
	location_temp = location_degree+location_min;
	return location_temp;
}
void strdel(char * str,char c)
{
	char *p = str;
	while(*str)
	{
		if(*str!=c)
			*p++ = *str;
		str++;
	}
	*p = '\0';
}